#pylint: skip-file
# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'data/ui/manager/ui_new_job_wizard.ui'
#
# Created by: PyQt5 UI code generator 5.7
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_NewJobWizard(object):
    def setupUi(self, NewJobWizard):
        NewJobWizard.setObjectName("NewJobWizard")
        NewJobWizard.resize(563, 436)
        self.page_repositories = QtWidgets.QWizardPage()
        self.page_repositories.setObjectName("page_repositories")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.page_repositories)
        self.verticalLayout.setObjectName("verticalLayout")
        self.listView = QtWidgets.QListView(self.page_repositories)
        self.listView.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.listView.setObjectName("listView")
        self.verticalLayout.addWidget(self.listView)
        NewJobWizard.addPage(self.page_repositories)
        self.page_sources = QtWidgets.QWizardPage()
        self.page_sources.setObjectName("page_sources")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.page_sources)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.tableView = QtWidgets.QTableView(self.page_sources)
        self.tableView.setAlternatingRowColors(True)
        self.tableView.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tableView.setShowGrid(False)
        self.tableView.setWordWrap(False)
        self.tableView.setObjectName("tableView")
        self.tableView.verticalHeader().setVisible(False)
        self.verticalLayout_2.addWidget(self.tableView)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.toolButton_2 = QtWidgets.QToolButton(self.page_sources)
        self.toolButton_2.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.toolButton_2.setObjectName("toolButton_2")
        self.horizontalLayout.addWidget(self.toolButton_2)
        self.toolButton_3 = QtWidgets.QToolButton(self.page_sources)
        self.toolButton_3.setObjectName("toolButton_3")
        self.horizontalLayout.addWidget(self.toolButton_3)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        NewJobWizard.addPage(self.page_sources)
        self.page_schedule = QtWidgets.QWizardPage()
        self.page_schedule.setObjectName("page_schedule")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.page_schedule)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.label = QtWidgets.QLabel(self.page_schedule)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.verticalLayout_4.addWidget(self.label)
        self.checkBox = QtWidgets.QCheckBox(self.page_schedule)
        self.checkBox.setObjectName("checkBox")
        self.verticalLayout_4.addWidget(self.checkBox)
        self.widget_2 = ScheduleWidget(self.page_schedule)
        self.widget_2.setObjectName("widget_2")
        self.verticalLayout_4.addWidget(self.widget_2)
        NewJobWizard.addPage(self.page_schedule)
        self.wizardPage = QtWidgets.QWizardPage()
        self.wizardPage.setObjectName("wizardPage")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.wizardPage)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_16 = QtWidgets.QLabel(self.wizardPage)
        self.label_16.setWordWrap(True)
        self.label_16.setObjectName("label_16")
        self.verticalLayout_3.addWidget(self.label_16)
        self.plainTextEdit = QtWidgets.QPlainTextEdit(self.wizardPage)
        font = QtGui.QFont()
        font.setFamily("Monospace")
        self.plainTextEdit.setFont(font)
        self.plainTextEdit.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
        self.plainTextEdit.setReadOnly(True)
        self.plainTextEdit.setObjectName("plainTextEdit")
        self.verticalLayout_3.addWidget(self.plainTextEdit)
        NewJobWizard.addPage(self.wizardPage)
        self.action_add_source = QtWidgets.QAction(NewJobWizard)
        self.action_add_source.setObjectName("action_add_source")

        self.retranslateUi(NewJobWizard)
        QtCore.QMetaObject.connectSlotsByName(NewJobWizard)

    def retranslateUi(self, NewJobWizard):
        _translate = QtCore.QCoreApplication.translate
        NewJobWizard.setWindowTitle(_translate("NewJobWizard", "Wizard"))
        self.page_repositories.setTitle(_translate("NewJobWizard", "Repositories"))
        self.page_repositories.setSubTitle(_translate("NewJobWizard", "Select the repositories where data will be saved."))
        self.page_sources.setTitle(_translate("NewJobWizard", "Sources"))
        self.page_sources.setSubTitle(_translate("NewJobWizard", "Select the directories to back up."))
        self.toolButton_2.setText(_translate("NewJobWizard", "..."))
        self.toolButton_3.setText(_translate("NewJobWizard", "..."))
        self.page_schedule.setTitle(_translate("NewJobWizard", "Schedule"))
        self.page_schedule.setSubTitle(_translate("NewJobWizard", "When and how often shall the job run?"))
        self.label.setText(_translate("NewJobWizard", "<html><head/><body><p><span style=\" font-weight:600;\">Note:</span> If you choose to use the global settings with the box below, changes to the global settings made <span style=\" font-style:italic;\">after</span> the job is created will be applied to the job, too. If you simply want to copy the global settings, uncheck the box below.</p></body></html>"))
        self.checkBox.setText(_translate("NewJobWizard", "Use global settings"))
        self.wizardPage.setTitle(_translate("NewJobWizard", "Finished!"))
        self.wizardPage.setSubTitle(_translate("NewJobWizard", "The configuration is finished."))
        self.label_16.setText(_translate("NewJobWizard", "<html><head/><body><p>If needed, you can make modifications to the job configuration below. In case of doubt, simply click continue.</p></body></html>"))
        self.action_add_source.setText(_translate("NewJobWizard", "Add source"))

from qtborg.schedule_widget import ScheduleWidget
