#pylint: skip-file
# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'data/ui/manager/ui_schedule_widget.ui'
#
# Created by: PyQt5 UI code generator 5.7
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(400, 300)
        self.formLayout = QtWidgets.QFormLayout(Form)
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(Form)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.spinBox = QtWidgets.QSpinBox(Form)
        self.spinBox.setMinimum(1)
        self.spinBox.setMaximum(10000)
        self.spinBox.setObjectName("spinBox")
        self.horizontalLayout.addWidget(self.spinBox)
        self.comboBox_2 = QtWidgets.QComboBox(Form)
        self.comboBox_2.setObjectName("comboBox_2")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.horizontalLayout.addWidget(self.comboBox_2)
        self.formLayout.setLayout(0, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout)
        self.label_2 = QtWidgets.QLabel(Form)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.comboBox = QtWidgets.QComboBox(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBox.sizePolicy().hasHeightForWidth())
        self.comboBox.setSizePolicy(sizePolicy)
        self.comboBox.setObjectName("comboBox")
        self.comboBox.addItem("")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.comboBox)
        self.checkBox = QtWidgets.QCheckBox(Form)
        self.checkBox.setObjectName("checkBox")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.SpanningRole, self.checkBox)
        self.label_3 = QtWidgets.QLabel(Form)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.spinBox_2 = QtWidgets.QSpinBox(Form)
        self.spinBox_2.setMinimum(1)
        self.spinBox_2.setMaximum(59)
        self.spinBox_2.setObjectName("spinBox_2")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.spinBox_2)
        self.label_4 = QtWidgets.QLabel(Form)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.spinBox_3 = QtWidgets.QSpinBox(Form)
        self.spinBox_3.setMinimum(1)
        self.spinBox_3.setMaximum(24)
        self.spinBox_3.setObjectName("spinBox_3")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.spinBox_3)
        self.label_5 = QtWidgets.QLabel(Form)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.label_5)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.spinBox_4 = QtWidgets.QSpinBox(Form)
        self.spinBox_4.setObjectName("spinBox_4")
        self.horizontalLayout_2.addWidget(self.spinBox_4)
        self.comboBox_3 = QtWidgets.QComboBox(Form)
        self.comboBox_3.setObjectName("comboBox_3")
        self.horizontalLayout_2.addWidget(self.comboBox_3)
        self.formLayout.setLayout(5, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout_2)
        self.label_6 = QtWidgets.QLabel(Form)
        self.label_6.setObjectName("label_6")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.label_6)
        self.spinBox_5 = QtWidgets.QSpinBox(Form)
        self.spinBox_5.setMinimum(1)
        self.spinBox_5.setMaximum(20000)
        self.spinBox_5.setObjectName("spinBox_5")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.spinBox_5)
        self.label_7 = QtWidgets.QLabel(Form)
        self.label_7.setObjectName("label_7")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.LabelRole, self.label_7)
        self.spinBox_6 = QtWidgets.QSpinBox(Form)
        self.spinBox_6.setMinimum(1)
        self.spinBox_6.setMaximum(200000)
        self.spinBox_6.setObjectName("spinBox_6")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.FieldRole, self.spinBox_6)

        self.retranslateUi(Form)
        self.comboBox.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label.setText(_translate("Form", "Run every"))
        self.comboBox_2.setItemText(0, _translate("Form", "Minutes"))
        self.comboBox_2.setItemText(1, _translate("Form", "Hours"))
        self.comboBox_2.setItemText(2, _translate("Form", "Days"))
        self.comboBox_2.setItemText(3, _translate("Form", "Weeks"))
        self.comboBox_2.setItemText(4, _translate("Form", "Months"))
        self.comboBox_2.setItemText(5, _translate("Form", "Years"))
        self.label_2.setText(_translate("Form", "Timezone"))
        self.comboBox.setItemText(0, _translate("Form", "Local time"))
        self.checkBox.setText(_translate("Form", "Run job at specific time in the interval"))
        self.label_3.setText(_translate("Form", "Minute"))
        self.label_4.setText(_translate("Form", "Hour"))
        self.label_5.setText(_translate("Form", "Day of week"))
        self.spinBox_4.setToolTip(_translate("Form", "If the interval is longer than one week, select the week to run the job at here."))
        self.label_6.setText(_translate("Form", "Day of month"))
        self.spinBox_5.setToolTip(_translate("Form", "If the interval has less days than the day at which the job is scheduled, the job is scheduled on the last day of the interval."))
        self.label_7.setText(_translate("Form", "Day of year"))
        self.spinBox_6.setToolTip(_translate("Form", "If the interval has less days than the day at which the job is scheduled, the job is scheduled on the last day of the interval."))

