#pylint: skip-file
# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'data/ui/manager/ui_prune_widget.ui'
#
# Created by: PyQt5 UI code generator 5.7
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_PruneWidget(object):
    def setupUi(self, PruneWidget):
        PruneWidget.setObjectName("PruneWidget")
        PruneWidget.resize(430, 246)
        self.gridLayout_2 = QtWidgets.QGridLayout(PruneWidget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.prune_monthly_enable = QtWidgets.QCheckBox(PruneWidget)
        self.prune_monthly_enable.setObjectName("prune_monthly_enable")
        self.gridLayout.addWidget(self.prune_monthly_enable, 3, 0, 1, 1)
        self.prune_yearly_enable = QtWidgets.QCheckBox(PruneWidget)
        self.prune_yearly_enable.setObjectName("prune_yearly_enable")
        self.gridLayout.addWidget(self.prune_yearly_enable, 4, 0, 1, 1)
        self.prune_hourly_enable = QtWidgets.QCheckBox(PruneWidget)
        self.prune_hourly_enable.setObjectName("prune_hourly_enable")
        self.gridLayout.addWidget(self.prune_hourly_enable, 0, 0, 1, 1)
        self.prune_daily_enable = QtWidgets.QCheckBox(PruneWidget)
        self.prune_daily_enable.setObjectName("prune_daily_enable")
        self.gridLayout.addWidget(self.prune_daily_enable, 1, 0, 1, 1)
        self.prune_weekly_narchives = QtWidgets.QSpinBox(PruneWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.prune_weekly_narchives.sizePolicy().hasHeightForWidth())
        self.prune_weekly_narchives.setSizePolicy(sizePolicy)
        self.prune_weekly_narchives.setObjectName("prune_weekly_narchives")
        self.gridLayout.addWidget(self.prune_weekly_narchives, 2, 1, 1, 1)
        self.prune_monthly_narchives = QtWidgets.QSpinBox(PruneWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.prune_monthly_narchives.sizePolicy().hasHeightForWidth())
        self.prune_monthly_narchives.setSizePolicy(sizePolicy)
        self.prune_monthly_narchives.setObjectName("prune_monthly_narchives")
        self.gridLayout.addWidget(self.prune_monthly_narchives, 3, 1, 1, 1)
        self.prune_weekly_enable = QtWidgets.QCheckBox(PruneWidget)
        self.prune_weekly_enable.setObjectName("prune_weekly_enable")
        self.gridLayout.addWidget(self.prune_weekly_enable, 2, 0, 1, 1)
        self.prune_yearly_narchives = QtWidgets.QSpinBox(PruneWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.prune_yearly_narchives.sizePolicy().hasHeightForWidth())
        self.prune_yearly_narchives.setSizePolicy(sizePolicy)
        self.prune_yearly_narchives.setObjectName("prune_yearly_narchives")
        self.gridLayout.addWidget(self.prune_yearly_narchives, 4, 1, 1, 1)
        self.prune_daily_narchives = QtWidgets.QSpinBox(PruneWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.prune_daily_narchives.sizePolicy().hasHeightForWidth())
        self.prune_daily_narchives.setSizePolicy(sizePolicy)
        self.prune_daily_narchives.setObjectName("prune_daily_narchives")
        self.gridLayout.addWidget(self.prune_daily_narchives, 1, 1, 1, 1)
        self.prune_hourly_narchives = QtWidgets.QSpinBox(PruneWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.prune_hourly_narchives.sizePolicy().hasHeightForWidth())
        self.prune_hourly_narchives.setSizePolicy(sizePolicy)
        self.prune_hourly_narchives.setObjectName("prune_hourly_narchives")
        self.gridLayout.addWidget(self.prune_hourly_narchives, 0, 1, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 1, 0, 1, 3)
        self.prune_keep_within_enable = QtWidgets.QCheckBox(PruneWidget)
        self.prune_keep_within_enable.setObjectName("prune_keep_within_enable")
        self.gridLayout_2.addWidget(self.prune_keep_within_enable, 2, 0, 1, 1)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.prune_keep_within_amount = QtWidgets.QSpinBox(PruneWidget)
        self.prune_keep_within_amount.setMinimum(1)
        self.prune_keep_within_amount.setMaximum(10000)
        self.prune_keep_within_amount.setObjectName("prune_keep_within_amount")
        self.horizontalLayout_3.addWidget(self.prune_keep_within_amount)
        self.prune_keep_within_unit = QtWidgets.QComboBox(PruneWidget)
        self.prune_keep_within_unit.setObjectName("prune_keep_within_unit")
        self.prune_keep_within_unit.addItem("")
        self.prune_keep_within_unit.addItem("")
        self.prune_keep_within_unit.addItem("")
        self.prune_keep_within_unit.addItem("")
        self.prune_keep_within_unit.addItem("")
        self.prune_keep_within_unit.addItem("")
        self.horizontalLayout_3.addWidget(self.prune_keep_within_unit)
        self.gridLayout_2.addLayout(self.horizontalLayout_3, 2, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(17, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem, 2, 2, 1, 1)
        self.prune_enable = QtWidgets.QCheckBox(PruneWidget)
        self.prune_enable.setObjectName("prune_enable")
        self.gridLayout_2.addWidget(self.prune_enable, 0, 0, 1, 2)

        self.retranslateUi(PruneWidget)
        self.prune_keep_within_unit.setCurrentIndex(2)
        QtCore.QMetaObject.connectSlotsByName(PruneWidget)

    def retranslateUi(self, PruneWidget):
        _translate = QtCore.QCoreApplication.translate
        PruneWidget.setWindowTitle(_translate("PruneWidget", "Form"))
        self.prune_monthly_enable.setText(_translate("PruneWidget", "Monthly"))
        self.prune_yearly_enable.setText(_translate("PruneWidget", "Yearly"))
        self.prune_hourly_enable.setText(_translate("PruneWidget", "Hourly"))
        self.prune_daily_enable.setText(_translate("PruneWidget", "Daily"))
        self.prune_weekly_enable.setText(_translate("PruneWidget", "Weekly"))
        self.prune_keep_within_enable.setText(_translate("PruneWidget", "Unconditionally keep archives younger than"))
        self.prune_keep_within_unit.setItemText(0, _translate("PruneWidget", "Minutes"))
        self.prune_keep_within_unit.setItemText(1, _translate("PruneWidget", "Hours"))
        self.prune_keep_within_unit.setItemText(2, _translate("PruneWidget", "Days"))
        self.prune_keep_within_unit.setItemText(3, _translate("PruneWidget", "Weeks"))
        self.prune_keep_within_unit.setItemText(4, _translate("PruneWidget", "Months"))
        self.prune_keep_within_unit.setItemText(5, _translate("PruneWidget", "Years"))
        self.prune_enable.setText(_translate("PruneWidget", "Enable pruning"))

